@@ -4,26 +4,26 @@ pub use std::iter::FromIterator;
 mod r#async;
 mod gen;
 mod guid;
+mod import_limit;
 mod iterator;
 mod object;
 mod parser;
 mod squote;
 pub mod tables;
 mod to_ident;
-mod type_limits;
 mod type_tree;
 pub mod types;
 mod workspace;
 
 pub use gen::*;
 pub use guid::*;
+pub use import_limit::*;
 pub use iterator::*;
 pub use object::*;
 pub use parser::*;
 pub use r#async::*;
 pub use squote::*;
 pub use to_ident::*;
-pub use type_limits::*;
 pub use type_tree::*;
 pub use workspace::*;
 
