@@ -118,7 +118,9 @@ impl MethodSignature {
             return quote! {};
         }
 
-        let params = if interface.kind == InterfaceKind::Composable {
+        let params = if interface.kind == InterfaceKind::Composable
+            || interface.kind == InterfaceKind::Extend
+        {
             &self.params[..self.params.len() - 2]
         } else {
             &self.params
@@ -157,12 +159,14 @@ impl MethodSignature {
 
         // The ABI obviously still has the two composable parameters. Here we just pass the default in and out
         // arguments to ensure the call succeeds in the non-aggregating case.
-        let composable_args = if interface.kind == InterfaceKind::Composable {
-            quote! {
+        let composable_args = match interface.kind {
+            InterfaceKind::Composable => quote! {
                 ::std::ptr::null_mut(), ::windows::Abi::set_abi(&mut ::std::option::Option::<::windows::IInspectable>::None),
-            }
-        } else {
-            quote! {}
+            },
+            InterfaceKind::Extend => quote! {
+                ::windows::Abi::abi(&derived__), ::windows::Abi::set_abi(base__),
+            },
+            _ => quote! {},
         };
 
         let vcall = if let Some(return_type) = &self.return_type {
@@ -220,11 +224,24 @@ impl MethodSignature {
                     }
                 }
             }
+            InterfaceKind::Extend => {
+                let interface_name = to_ident(interface.def.name());
+                quote! {
+                    pub fn #name<#constraints>(self, #params) -> ::windows::Result<#return_type_tokens> {
+                        unsafe {
+                            let (derived__, base__) = ::windows::Compose::compose(self);
+                            #return_type_tokens::#interface_name(|this| unsafe { #vcall })
+                        }
+                    }
+                }
+            }
         }
     }
 
     fn gen_name(&self, method: &MethodInfo, interface: &InterfaceInfo) -> Ident {
-        if interface.kind == InterfaceKind::Composable && self.params.len() == 2 {
+        if (interface.kind == InterfaceKind::Composable || interface.kind == InterfaceKind::Extend)
+            && self.params.len() == 2
+        {
             format_ident!("new")
         } else if method.overload > 1 {
             format_ident!("{}{}", &method.name, method.overload)
@@ -368,8 +385,6 @@ impl MethodParam {
         } else if self.param.is_input() {
             if self.signature.kind.is_primitive() {
                 quote! { #name }
-            } else if let ElementType::Enum(_) = self.signature.kind {
-                quote! { #name }
             } else if self.is_const() {
                 quote! { &*(#name as *const <#kind as ::windows::Abi>::Abi as *const <#kind as ::windows::RuntimeType>::DefaultType) }
             } else {
@@ -451,7 +466,7 @@ impl MethodParam {
             }
         }
 
-        tokens.combine(&self.signature.kind.gen_abi_name(gen));
+        tokens.combine(&self.signature.kind.gen_abi_type(gen));
         tokens
     }
 
@@ -466,7 +481,7 @@ impl MethodParam {
             }
         }
 
-        tokens.combine(&self.signature.kind.gen_abi_name(gen));
+        tokens.combine(&self.signature.kind.gen_abi_type(gen));
         tokens
     }
 
