@@ -6,7 +6,6 @@ mod constant_value;
 mod element_type;
 mod field_flags;
 mod file;
-mod generic_type;
 mod interface_info;
 mod interface_kind;
 mod method_flags;
@@ -27,7 +26,6 @@ pub use constant_value::*;
 pub use element_type::*;
 pub use field_flags::*;
 pub use file::*;
-pub use generic_type::*;
 pub use interface_info::*;
 pub use interface_kind::*;
 pub use method_flags::*;
