@@ -6,22 +6,48 @@ pub fn gen_bstr() -> TokenStream {
         #[derive(::std::cmp::Eq)]
         pub struct BSTR(*mut u16);
         impl BSTR {
+            /// Create an empty `BSTR`.
+            ///
+            /// This function does not allocate memory.
+            pub fn new() -> Self {
+                Self(std::ptr::null_mut())
+            }
+
+            /// Returns `true` if the string is empty.
             pub fn is_empty(&self) -> bool {
                 self.0.is_null()
             }
-            fn from_wide(value: &[u16]) -> Self {
+
+            /// Returns the length of the string.
+            pub fn len(&self) -> usize {
+                if self.is_empty() {
+                    return 0;
+                }
+
+                unsafe { super::System::OleAutomation::SysStringLen(self) as usize }
+            }
+
+            /// Create a `BSTR` from a slice of 16-bit characters.
+            pub fn from_wide(value: &[u16]) -> Self {
                 if value.len() == 0 {
                     return Self(::std::ptr::null_mut());
                 }
 
-                unsafe { SysAllocStringLen(super::SystemServices::PWSTR(value.as_ptr() as _), value.len() as u32) }
+                unsafe {
+                    super::System::OleAutomation::SysAllocStringLen(
+                        PWSTR(value.as_ptr() as _),
+                        value.len() as u32,
+                    )
+                }
             }
-            fn as_wide(&self) -> &[u16] {
+
+            /// Get the string as 16-bit characters.
+            pub fn as_wide(&self) -> &[u16] {
                 if self.0.is_null() {
                     return &[];
                 }
 
-                unsafe { ::std::slice::from_raw_parts(self.0 as *const u16, SysStringLen(self) as usize) }
+                unsafe { ::std::slice::from_raw_parts(self.0 as *const u16, self.len()) }
             }
         }
         impl ::std::clone::Clone for BSTR {
@@ -110,7 +136,7 @@ pub fn gen_bstr() -> TokenStream {
         impl ::std::ops::Drop for BSTR {
             fn drop(&mut self) {
                 if !self.0.is_null() {
-                    unsafe { SysFreeString(self as &Self); }
+                    unsafe { super::System::OleAutomation::SysFreeString(self as &Self) }
                 }
             }
         }
