@@ -4,47 +4,9 @@ use super::*;
 pub struct Enum(pub tables::TypeDef);
 
 impl Enum {
-    pub fn definition(&self) -> Vec<ElementType> {
-        vec![ElementType::Enum(self.clone())]
-    }
-
-    pub fn type_signature(&self) -> String {
-        let underlying_type = match self.underlying_type() {
-            ElementType::I32 => "i4",
-            ElementType::U32 => "u4",
-            _ => unexpected!(),
-        };
-
-        format!(
-            "enum({}.{};{})",
-            self.0.namespace(),
-            self.0.name(),
-            underlying_type
-        )
-    }
-
-    pub fn underlying_type(&self) -> ElementType {
-        if let Some(field) = self.0.fields().next() {
-            if let Some(constant) = field.constant() {
-                return constant.value_type();
-            } else {
-                let blob = &mut field.blob();
-                blob.read_unsigned();
-                blob.read_modifiers();
-
-                blob.read_expected(0x1D);
-                blob.read_modifiers();
-
-                return ElementType::from_blob(blob, &[]);
-            }
-        }
-
-        unexpected!();
-    }
-
     pub fn gen(&self, gen: &Gen) -> TokenStream {
         let name = self.0.gen_name(gen);
-        let underlying_type = self.underlying_type();
+        let underlying_type = self.0.underlying_type();
 
         // WinRT enums don't have the flags attribute but are paritioned merely based
         // on whether they are signed.
@@ -86,48 +48,55 @@ impl Enum {
         let underlying_type = underlying_type.gen_name(gen);
         let mut last: Option<ConstantValue> = None;
 
-        let fields = if self.0.is_scoped() {
-            let fields = self.0.fields().filter_map(|field| {
-                if field.flags().literal() {
-                    let name = to_ident(field.name());
+        let fields = self.0.fields().filter_map(|field| {
+            // TODO: workaround for https://github.com/microsoft/win32metadata/issues/522
+            if field.name() == self.0.name() {
+                return None;
+            }
 
-                    if let Some(constant) = field.constant() {
-                        let value = constant.value().gen_value();
+            if field.flags().literal() {
+                let field_name = to_ident(field.name());
 
-                        Some(quote! {
-                            pub const #name: Self = Self(#value);
-                        })
-                    } else if let Some(last_value) = &last {
-                        let next = last_value.next();
-                        let value = next.gen_value();
-                        last = Some(next);
+                if let Some(constant) = field.constant() {
+                    let value = constant.value().gen_value();
 
-                        Some(quote! {
-                            pub const #name: Self = Self(#value);
-                        })
-                    } else {
-                        last = Some(ConstantValue::I32(0));
+                    Some(quote! {
+                        pub const #field_name: #name = #name(#value);
+                    })
+                } else if let Some(last_value) = &last {
+                    let next = last_value.next();
+                    let value = next.gen_value();
+                    last = Some(next);
 
-                        Some(quote! {
-                            pub const #name: Self = Self(0);
-                        })
-                    }
+                    Some(quote! {
+                        pub const #field_name: #name = #name(#value);
+                    })
                 } else {
-                    None
-                }
-            });
+                    last = Some(ConstantValue::I32(0));
 
+                    Some(quote! {
+                        pub const #field_name: #name = #name(0);
+                    })
+                }
+            } else {
+                None
+            }
+        });
+
+        let fields = if self.0.is_scoped() {
             quote! {
                 impl #name {
                     #(#fields)*
                 }
             }
         } else {
-            quote! {}
+            quote! {
+                #(#fields)*
+            }
         };
 
         let runtime_type = if self.0.is_winrt() {
-            let signature = Literal::byte_string(&self.type_signature().as_bytes());
+            let signature = Literal::byte_string(&self.0.type_signature().as_bytes());
 
             quote! {
                 unsafe impl ::windows::RuntimeType for #name {
@@ -164,7 +133,7 @@ mod tests {
 
     #[test]
     fn test_signature() {
-        let t = TypeReader::get_enum("Windows.Foundation", "AsyncStatus");
+        let t = TypeReader::get().resolve_type_def("Windows.Foundation", "AsyncStatus");
         assert_eq!(
             t.type_signature(),
             "enum(Windows.Foundation.AsyncStatus;i4)"
