@@ -4,19 +4,10 @@ use super::*;
 pub struct Function(pub tables::MethodDef);
 
 impl Function {
-    pub fn gen_name(&self, gen: &Gen) -> TokenStream {
-        let namespace = gen.namespace(self.0.parent().namespace());
-        let name = format_ident!("{}", self.0.name());
-        quote! { #namespace #name }
-    }
-
-    pub fn dependencies(&self) -> Vec<ElementType> {
-        self.0.dependencies(&[])
-    }
-
-    pub fn gen(&self, gen: &Gen) -> TokenStream {
-        let name = self.gen_name(gen);
-        let signature = self.0.signature(&[]);
+    // TODO: move to MethodDef?
+    pub fn gen(def: &tables::MethodDef, gen: &Gen) -> TokenStream {
+        let name = def.gen_name(gen);
+        let signature = def.signature(&[]);
 
         let constraints = signature.gen_constraints(&signature.params);
         let params = signature.gen_win32_params(&signature.params, gen);
@@ -44,37 +35,43 @@ impl Function {
 
         let args = signature.params.iter().map(|p| p.gen_win32_abi_arg());
 
-        let mut link = self.0.impl_map().expect("Function").scope().name();
+        let mut link = def.impl_map().expect("Function").scope().name();
 
         // TODO: workaround for https://github.com/microsoft/windows-rs/issues/463
         if link.contains("-ms-win-") || link == "D3DCOMPILER_47" || link == "SspiCli" {
             link = "onecoreuap";
         }
 
-        let body = if cfg!(windows) {
-            if signature.has_query_interface() {
-                let leading_params = &signature.params[..signature.params.len() - 2];
-                let args = leading_params.iter().map(|p| p.gen_win32_abi_arg());
+        let body = if signature.has_query_interface() {
+            let leading_params = &signature.params[..signature.params.len() - 2];
+            let args = leading_params.iter().map(|p| p.gen_win32_abi_arg());
 
-                quote! {
-                    #[link(name = #link)]
-                    extern "system" {
-                        fn #name(#(#abi_params),*) #abi_return_type;
-                    }
-                    let mut result__ = ::std::option::Option::None;
-                    #name(#(#args,)* &<T as ::windows::Interface>::IID, ::windows::Abi::set_abi(&mut result__)).and_some(result__)
-                }
-            } else {
-                quote! {
-                    #[link(name = #link)]
-                    extern "system" {
-                        fn #name(#(#abi_params),*) #abi_return_type;
-                    }
-                    #name(#(#args),*)
+            quote! {
+                #[link(name = #link)]
+                extern "system" {
+                    fn #name(#(#abi_params),*) #abi_return_type;
                 }
+                let mut result__ = ::std::option::Option::None;
+                #name(#(#args,)* &<T as ::windows::Interface>::IID, ::windows::Abi::set_abi(&mut result__)).and_some(result__)
             }
         } else {
             quote! {
+                #[link(name = #link)]
+                extern "system" {
+                    fn #name(#(#abi_params),*) #abi_return_type;
+                }
+                #name(#(#args),*)
+            }
+        };
+
+        // Don't link on windows dlls when generating code for non-windows:
+        let body = quote! {
+            #[cfg(windows)]
+            {
+                #body
+            }
+            #[cfg(not(windows))]
+            {
                 unimplemented!("Unsupported target OS");
             }
         };
